<?php
/**
 * @brief		Dashboard extension: forumStats
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Spacious Widgets
 * @since		16 Sep 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\spaciouswidgets\extensions\core\Dashboard;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Dashboard extension: forumStats
 */
class _forumStats
{
	/**
	 * Can the current user view this dashboard item?
	 *
	 * @return    bool
	 */
	public function canView()
	{
		return TRUE;
	}

	/**
	 * Return the block HTML show on the dashboard
	 *
	 * @return    string
	 */
	public function getBlock()
	{

		/* Setup for reusable stuff, arrays, etc... */

		$daytime = \IPS\DateTime::create()->sub( new \DateInterval( 'PT24H' ) )->getTimeStamp();
		$today = (new\IPS\DateTime())->setTime(0,0);
		$today = $today->getTimeStamp();

		$weektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P7D' ) )->getTimeStamp();
		$twoweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P14D' ) )->getTimeStamp();
		$threeweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P21D' ) )->getTimeStamp();
		$fourweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P28D' ) )->getTimeStamp();
		$fiveweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P35D' ) )->getTimeStamp();
		$sixweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P42D' ) )->getTimeStamp();
		$sevenweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P49D' ) )->getTimeStamp();
		$eightweektime = \IPS\DateTime::create()->sub( new \DateInterval( 'P56D' ) )->getTimeStamp();

		$fstats = [];


		/* Count of categories and forums */
		/* Cats determined by sub_can_post being 0 and that's about it */

		$fstats['forums_count'] =
			\IPS\Db::i()->select( "COUNT(*)", 'forums_forums', [ 'sub_can_post = ?', 1 ] )->first();
		$fstats['categories_count'] =
			\IPS\Db::i()->select( "COUNT(*)", 'forums_forums', [ 'sub_can_post = ?', 0 ] )->first();


		/* Forums with most topics, posts - these are pretty low weight plus topic and posts counts per */
		/* are already calculated and stored automatically so just select and go */
		/* Maybe expand this to show top three forums with most topics, posts or save that for other thing... */

		$most_topics =
			\IPS\Db::i()->select( '*', 'forums_forums', [ 'sub_can_post = ?', 1 ], 'topics DESC', 1 )->first();
		$most_posts = \IPS\Db::i()->select( '*', 'forums_forums', [ 'sub_can_post = ?', 1 ], 'posts DESC', 1 )->first();
		$forum_mt = \IPS\forums\Forum::loadAndCheckPerms( $most_topics['id'] );
		$forum_mtc = $most_topics['topics'];
		$forum_mp = \IPS\forums\Forum::loadAndCheckPerms( $most_posts['id'] );
		$forum_mpc = $most_posts['posts'];


		// Topics Started, 90 Days.
		$start = new \DateTime();
		$start->sub(new \DateInterval('P90D'));
		$end = new \DateTime();
		$end->sub(new \DateInterval('P1D'));
		$end->setTime(23,59,59); // end is now yesterday, but at 11:59:59 pm
		$interval = new \DateInterval('P1D');
		$daterange = new \DatePeriod($start, $interval, $end);

		$ninety_days_topics = \IPS\Db::i()->select(
			"DATE_FORMAT( FROM_UNIXTIME( forums_topics.start_date ), '%Y-%m-%d') AS day, COUNT(*) AS NumTopics",
			'forums_topics',
			[[ 'forums_topics.start_date >= ?', \IPS\DateTime::create()->sub( new \DateInterval( 'P91D' ) )->getTimeStamp() ]],
			NULL,
			NULL,
			'day'
		);

		$ndt = [];
		foreach ( $ninety_days_topics as $thing )
		{
			$ndt[$thing['day']] = $thing['NumTopics'];
		}

		$ninety_days_topics = [];

		// If there is no match, enter 0 as value, otherwise pull from data.
		foreach($daterange as $dt)
		{
			if(array_key_exists($dt->format("Y-m-d"), $ndt))
			{
				$ninety_days_topics[$dt->format("Y-m-d")] = $ndt[$dt->format("Y-m-d")];
			}
			else
			{
				$ninety_days_topics[$dt->format("Y-m-d")] = 0;
			}
		}

		// Slice back for just 7 days.
		$seven_days_topics = array_slice($ninety_days_topics, -7, 7, TRUE);
		$fstats['seven_days_topics_sum'] = array_sum($seven_days_topics);
		// Now compress to an every 3 days total version. This will give us 18 data points
		// instead of 90, for people wanting a more subtle graph and trend line.
		$ninetybyfives = array_chunk($ninety_days_topics, 3, true);

		$count = 1;
		$nbf_topics = [];
		foreach ($ninetybyfives as $fivers)
		{
			$nbf_topics[current(array_keys($fivers))] = array_sum($fivers);
			$count++;
		}



		// Responses Made, 90 Days. Already have the daterange stuff set
		$ninety_days_responses = \IPS\Db::i()->select(
			"DATE_FORMAT( FROM_UNIXTIME( forums_posts.post_date ), '%Y-%m-%d') AS day, COUNT(*) AS NumResponses",
			'forums_posts',
			[[ 'forums_posts.post_date >= ?', \IPS\DateTime::create()->sub( new \DateInterval( 'P91D' ) )->getTimeStamp()], [ 'new_topic = ?', 0 ]],
			NULL,
			NULL,
			'day'
		);

		$ndr = [];
		foreach ( $ninety_days_responses as $thing )
		{
			$ndr[$thing['day']] = $thing['NumResponses'];
		}

		$ninety_days_responses = [];

		// If there is no match, enter 0 as value, otherwise pull from data.
		foreach($daterange as $dt)
		{
			if(array_key_exists($dt->format("Y-m-d"), $ndr))
			{
				$ninety_days_responses[$dt->format("Y-m-d")] = $ndr[$dt->format("Y-m-d")];
			}
			else
			{
				$ninety_days_responses[$dt->format("Y-m-d")] = 0;
			}
		}

		// Slice back for just 7 days.
		$seven_days_responses = array_slice($ninety_days_responses, -7, 7, TRUE);
		$fstats['seven_days_responses_sum'] = array_sum($seven_days_responses);


		// Now compress to an every 3 days total version. This will give us 18 data points
		// instead of 90, for people wanting a more subtle graph and trend line.
		$ninetybyfives = array_chunk($ninety_days_responses, 3, true);
		$count = 1;
		$nbf_responses = [];
		foreach ($ninetybyfives as $fivers)
		{
			$nbf_responses[current(array_keys($fivers))] = array_sum($fivers);
			$count++;
		}

		// Sum and percentages for bars
		$fstats['seven_days_forums_total'] = $fstats['seven_days_responses_sum'] + $fstats['seven_days_topics_sum'];

		if ($fstats['seven_days_forums_total'] != 0)
		{
			$fstats['seven_days_responses_percent'] = round((($fstats['seven_days_responses_sum'] / $fstats['seven_days_forums_total']))*100, 1);
			$fstats['seven_days_topics_percent'] = round(100 - $fstats['seven_days_responses_percent'], 1);
		}
		else
		{
			$fstats['seven_days_topics_percent'] = 50;
			$fstats['seven_days_responses_percent'] = 50;
		}

		/* Total topics and posts for entire community */

		$fstats['topics_total'] = \IPS\Db::i()->select( "COUNT(*)", 'forums_topics', [ 'approved = ?', 1 ] )->first();
		$fstats['responses_total'] =
			\IPS\Db::i()->select( "COUNT(*)", 'forums_posts', [ 'new_topic = ?', 0 ] )->first();


		/* Topics created since midnight */

		$fstats['24h_topics'] = \IPS\Db::i()->select(
			'COUNT(*)', 'forums_topics', [ [ 'forums_topics.start_date >= ?', \IPS\DateTime::create()->setTime(0,0)->getTimeStamp() ], [ 'approved = ?', 1 ] ]
		)->first();
		$fstats['7d_topics'] = \IPS\Db::i()->select(
			'COUNT(*)', 'forums_topics', [ [ 'forums_topics.start_date > ?', $weektime ], [ 'approved = ?', 1 ] ]
		)->first();

		/* Responses created since midnight */
		/* Note this is philosophically different than usual as I will be subtracting first posts from the number. I want responses! */
		/* The new topic flag in the where array is what kills the topic first post stuff */

		$fstats['24h_responses'] = \IPS\Db::i()->select(
			'COUNT(*)', 'forums_posts', [ [ 'forums_posts.post_date >= ?', \IPS\DateTime::create()->setTime(0,0)->getTimeStamp() ], [ 'new_topic = ?', 0 ] ]
		)->first();
		$fstats['7d_responses'] = \IPS\Db::i()->select(
			'COUNT(*)', 'forums_posts', [ [ 'forums_posts.post_date > ?', $weektime ], [ 'new_topic = ?', 0 ] ]
		)->first();

		/* All-Times: Members that what started the most topics, most responses */

		$topic_starters = \IPS\Db::i()->select(
			'starter_id, COUNT(*) AS howmanytopics', 'forums_topics', NULL, 'howmanytopics DESC', 5, 'starter_id'
		);

		$count = 0;
		foreach ( $topic_starters as $thing )
		{
			$topicstarters[ $count ] = $thing;
			$count++;
		}

		$responds = \IPS\Db::i()->select(
			'author_id, COUNT(*) AS howmanyresponses', 'forums_posts', 'new_topic=0', 'howmanyresponses DESC', 5,
			'author_id'
		);

		$count = 0;
		$responders = [];
		foreach ( $responds as $thing )
		{
			$responders[$count] = $thing;
			$count++;
		}


		/* FIREY THINGS */


		/* Responders on fire - which can't possibly be interpreted in other ways... last 7 days */

		$firey_posters = \IPS\Db::i()->select(
			'author_id, COUNT(*) AS firey', 'forums_posts',
			[ [ 'forums_posts.post_date > ?', $weektime ], [ 'new_topic = ?', 0 ] ], 'firey DESC', 5, 'author_id'
		);

		/* Topic Makers on fire... last 7 days */

		$firey_topic_makers = \IPS\Db::i()->select(
			'author_id, COUNT(*) AS firey', 'forums_posts',
			[ [ 'forums_posts.post_date > ?', $weektime ], [ 'new_topic = ?', 1 ] ], 'firey DESC', 5, 'author_id'
		);

		/* Percentages */

		$firey_posters_total = 0;
		$firey_tm_total = 0;
		$firey_tm_percent = 0;
		$firey_posters_percent = 0;


		foreach ( $firey_topic_makers as $firey )
		{
			$firey_tm_total = $firey_tm_total + $firey['firey'];
		}

		if ( $firey_tm_total != 0 && $fstats['7d_responses'] != 0 )
		{
			$firey_tm_percent = ( $firey_tm_total / $fstats['7d_topics'] ) * 100;
			if ( is_nan( $firey_tm_percent ) )
			{
				$firey_tm_percent = 0;
			}
		}
		else
		{
			$firey_tm_percent = 0;
		}


		foreach ( $firey_posters as $firey )
		{
			$firey_posters_total = $firey_posters_total + $firey['firey'];
		}

		if ( $firey_posters_total != 0 && $fstats['7d_responses'] != 0 )
		{
			$firey_posters_percent = ( $firey_posters_total / $fstats['7d_responses'] ) * 100;
			if ( is_nan( $firey_posters_percent ) )
			{
				$firey_posters_percent = 0;
			}
		}
		else
		{
			$firey_posters_percent = 0;
		}




		// Set Chosen Graph Format
		if(\IPS\Settings::i()->spacious_graph_format == 0)
		{
			$graph_type = 'AreaChart';
		}
		else
		{
			$graph_type = 'LineChart';
		}

		// Set Chart Line Color
	    if (\IPS\Settings ::i()->spacious_donutcolor == 1)
	    {
		    $graphline_color = \IPS\Settings::i()->spacious_donut_primarycolor;
	    }
		else
		{
			$graphline_color = \IPS\Settings ::i()->sacp_header_background;
		}


		/* Topics 90 Days Chart */
		$chart = new \IPS\Helpers\Chart;
		$chart->addHeader( "Day", 'date' );
		$chart->addHeader( "Topics Started", 'number' );
		if(\IPS\Settings::i()->spacious_ft_graphs == 1)
		{
			foreach ( $nbf_topics as $k => $v )
			{
				$date = new \IPS\DateTime($k);
				$date->add(new \DateInterval('P1D'));
				$chart->addRow( [ $date, $v ] );
			}
		}
		else
		{
			foreach ( $ninety_days_topics as $k => $v )
			{
				$date = new \IPS\DateTime($k);
				$date->add(new \DateInterval('P1D'));
				$chart->addRow( [ $date, $v ] );
			}
		}
		$nineties_topics_chart = $chart->render(
			$graph_type, [
				'backgroundColor'     => 'transparent',
				'enableInteractivity' => 'true',
				'legend'              => [ 'position' => 'none' ],
				'lineWidth'           => \IPS\Settings::i()->spacious_graph_line_size,
				'height' => '120',
				'colors' => [$graphline_color],
				'chartArea'           => [
					'backgroundColor' => 'transparent',
					'stroke'          => '#f5f5f5',
					'width' => '100%',
					'height' => '110',
				],
				'reverseCategories'   => 'false',

				'hAxis'               => [ 'gridlines' => [ 'count' => "0" ], 'baselineColor' => "transparent", 'viewWindowMode' => 'maximized'  ],
				'vAxis'               => [ 'gridlines' => [ 'count' => "0" ], 'baselineColor' => "transparent", 'viewWindowMode' => 'maximized' ],
				'trendlines'          => [
					'0' => [
						'type'            => 'linear',
						'color'           => \IPS\Settings::i()->spacious_graph_trendline_color,
						'lineWidth'       => \IPS\Settings::i()->spacious_graph_trendline_width,
						'opacity'         => '0.5',
						'showR2'          => 'false',
						'pointsVisible'   => 'false',
						'enableInteractivity' => 'false',
						'visibleInLegend' => 'false'
					]
				]
			]
		);

		/* Responses 90 Days Chart */
		$chart = new \IPS\Helpers\Chart;
		$chart->addHeader( "Day", 'date' );
		$chart->addHeader( "Responses Made", 'number' );
		if(\IPS\Settings::i()->spacious_ft_graphs == 1)
		{
			foreach ( $nbf_responses as $k => $v )
			{
				$date = new \IPS\DateTime($k);
				$date->add(new \DateInterval('P1D'));
				$chart->addRow( [ $date, $v ] );
			}
		}
		else
		{
			foreach ( $ninety_days_responses as $k => $v )
			{
				$date = new \IPS\DateTime($k);
				$date->add(new \DateInterval('P1D'));
				$chart->addRow( [ $date, $v ] );
			}
		}
		$nineties_responses_chart = $chart->render(
			$graph_type, [
				'backgroundColor'     => 'transparent',
				'enableInteractivity' => 'true',
				'legend'              => [ 'position' => 'none' ],
				'lineWidth'           => \IPS\Settings::i()->spacious_graph_line_size,
				'height' => '120',
				'colors' => [$graphline_color],
				'chartArea'           => [
						'backgroundColor' => 'transparent',
						'stroke'          => '#f5f5f5',
					'width' => '100%',
					'height' => '110',
				],
				'reverseCategories'   => 'false',
				'hAxis'               => [ 'gridlines' => [ 'count' => "0" ], 'baselineColor' => "transparent", 'viewWindowMode' => 'maximized' ],
				'vAxis'               => [ 'gridlines' => [ 'count' => "0" ], 'baselineColor' => "transparent", 'viewWindowMode' => 'maximized' ],
				'trendlines'          => [
					'0' => [
						'type'            => 'linear',
						'color'           => \IPS\Settings::i()->spacious_graph_trendline_color,
						'lineWidth'       => \IPS\Settings::i()->spacious_graph_trendline_width,
						'opacity'         => '0.5',
						'showR2'          => 'false',
						'pointsVisible'   => 'false',
						'enableInteractivity' => 'false',
						'visibleInLegend' => 'false'
					]
				]
			]
		);


			/* PUSH OUT THE TEMPLATE */

			\IPS\Output::i()->cssFiles = array_merge(\IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'spaciouswidgets_fs.css', 'spaciouswidgets', 'admin' ));
			return \IPS\Theme::i()->getTemplate( 'dashboard', 'spaciouswidgets' )->forumStatistics($firey_posters, $firey_topic_makers, $fstats, $firey_posters_total, $firey_tm_total, $firey_tm_percent, $firey_posters_percent, $seven_days_topics, $nineties_topics_chart, $seven_days_responses, $nineties_responses_chart);
		}
}