<?php
/**
 * @brief		Dashboard extension: galleryStats
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Spacious Widgets
 * @since		04 Sep 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\spaciouswidgets\extensions\core\Dashboard;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Dashboard extension: galleryStats
 */
class _galleryStats
{
	/**
	* Can the current user view this dashboard item?
	*
	* @return	bool
	*/
	public function canView()
	{
		// Check for Gallery installed, disabled is fine though.

		$installed = array_keys( \IPS\Application::applications() );
		if (in_array( 'gallery', $installed ))
		{
			return TRUE;
		}

	}

	/** 
	 * Return the block HTML show on the dashboard
	 *
	 * @return	string
	 */
	public function getBlock()
	{
		/* Just in case, we'll check for a gallery table even though the view check should suffice... */
		if ( \IPS\Db::i()->checkForTable( 'gallery_images' ) )
		{
			$installed = 1;

			/* Get top four viewed images. This gets them as objects so we can actually grab the front url and link them */
			try {
			foreach ( \IPS\Db::i()->select( '*', 'gallery_images', NULL, 'image_views DESC', 4 ) as $mostviewed )
			{
				$topfourimages[] = \IPS\gallery\Image::constructFromData( $mostviewed );
			}
		}
	catch ( \Exception $e )
			{
				$topfourimages = [];
			}

			/* Get four newest images. This gets them as objects so we can actually grab the front url and link them */
			try {
				foreach ( \IPS\Db::i()->select( '*', 'gallery_images', NULL, 'image_date DESC', 4 ) as $newest )
				{
					$newestfourimages[] = \IPS\gallery\Image::constructFromData( $newest );
				}
			}
			catch ( \Exception $e )
			{
				$newestfourimages = [];
			}

			/* Basic Statistics cribbed from IPS Gallery widget plus my today count */
			$stats = [
				'total_oi_size' => (int) \IPS\Db::i()->select( 'SUM(image_file_size)', 'gallery_images' )->first(),
				'image_count'   => (int) \IPS\Db::i()->select( 'COUNT(*)', 'gallery_images' )->first(),
				'view_count'    => (int) \IPS\Db::i()->select( 'SUM(image_views)', 'gallery_images' )->first(),
				'comment_count' => (int) \IPS\Db::i()->select( 'COUNT(*)', 'gallery_comments' )->first(),
				'cat_count'     => (int) \IPS\Db::i()->select( 'COUNT(*)', 'gallery_categories' )->first(),
				'album_count'   => (int) \IPS\Db::i()->select( 'COUNT(*)', 'gallery_albums' )->first(),
				'today_count'   => (int) \IPS\Db::i()->select( 'COUNT(*)', 'gallery_images', [ [ 'gallery_images.image_date >= ?', \IPS\DateTime::create()->setTime(0,0)->getTimeStamp() ], [ 'gallery_images.image_approved = ?', 1 ] ]
						)->first()
			];

			$stats["total_oi_size"] = \IPS\Output\Plugin\Filesize::humanReadableFilesize($stats["total_oi_size"]);

			// Chart for 90 days of image uploads
			$start = new \DateTime();
			$start->sub(new \DateInterval('P90D'));
			$end = new \DateTime();
			$end->sub(new \DateInterval('P1D'));
			$end->setTime(23,59,59); // end is now yesterday, but at 11:59:59 pm
			$interval = new \DateInterval('P1D');
			$daterange = new \DatePeriod($start, $interval, $end);

			$ninety_days_images = \IPS\Db::i()->select(
				"DATE_FORMAT( FROM_UNIXTIME( gallery_images.image_date ), '%Y-%m-%d') AS day, COUNT(*) AS NumImages",
				'gallery_images',
				[[ 'gallery_images.image_date >= ?', \IPS\DateTime::create()->sub( new \DateInterval( 'P91D' ) )->getTimeStamp() ]],
				NULL,
				NULL,
				'day'
			);

			$ndi = [];
			foreach ( $ninety_days_images as $thing )
			{
				$ndi[$thing['day']] = $thing['NumImages'];
			}

			$ninety_days_images = [];

			// If there is no match, enter 0 as value, otherwise pull from data.
			foreach($daterange as $dt)
			{
				if(array_key_exists($dt->format("Y-m-d"), $ndi))
				{
					$ninety_days_images[$dt->format("Y-m-d")] = $ndi[$dt->format("Y-m-d")];
				}
				else
				{
					$ninety_days_images[$dt->format("Y-m-d")] = 0;
				}
			}

			// Slice back for just 7 days.
			$seven_days_images = array_slice($ninety_days_images, -7, 7, TRUE);

			// Now compress to an every 3 days total version. This will give us 18 data points
			// instead of 90, for people wanting a more subtle graph and trend line.
			$ninetybyfives_images = array_chunk($ninety_days_images, 3, true);

			$nbf_images = [];
			foreach ($ninetybyfives_images as $fivers)
			{
				$nbi_topics[current(array_keys($fivers))] = array_sum($fivers);
			}

			// FUTURE: Set these as statics?
			// Set Chosen Graph Format
			if(\IPS\Settings::i()->spacious_graph_format == 0)
			{
				$graph_type = 'AreaChart';
			}
			else
			{
				$graph_type = 'LineChart';
			}

			// Set Chart Line Color
			if (\IPS\Settings ::i()->spacious_donutcolor == 1)
			{
				$graphline_color = \IPS\Settings::i()->spacious_donut_primarycolor;
			}
			else
			{
				$graphline_color = \IPS\Settings ::i()->sacp_header_background;
			}



			/* Images 90 Days Chart */
			$chart = new \IPS\Helpers\Chart;
			$chart->addHeader( "Day", 'date' );
			$chart->addHeader( "Images Uploaded", 'number' );
			if(\IPS\Settings::i()->spacious_gt_graphs == 1)
			{
				foreach ( $nbi_topics as $k => $v )
				{
					$date = new \IPS\DateTime($k);
					$date->add(new \DateInterval('P1D'));
					$chart->addRow( [ $date, $v ] );
				}
			}
			else
			{
				foreach ( $ninety_days_images as $k => $v )
				{
					$date = new \IPS\DateTime($k);
					$date->add(new \DateInterval('P1D'));
					$chart->addRow( [ $date, $v ] );
				}
			}
			$nineties_images_chart = $chart->render(
				$graph_type, [
					'title'	=> "Images Uploaded, 90 Days",
					'backgroundColor'     => 'transparent',
					'enableInteractivity' => 'true',
					'legend'              => [ 'position' => 'none' ],
					'lineWidth'           => \IPS\Settings::i()->spacious_graph_line_size,
					'height' => '120',
					'colors' => [$graphline_color],
					'chartArea'           => [
						'backgroundColor' => 'transparent',
						'stroke'          => '#f5f5f5',
						'width' => '100%',
						'height' => '110',
					],
					'reverseCategories'   => 'false',

					'hAxis'               => [ 'gridlines' => [ 'count' => "0" ], 'baselineColor' => "transparent", 'viewWindowMode' => 'maximized'  ],
					'vAxis'               => [ 'gridlines' => [ 'count' => "0" ], 'baselineColor' => "transparent", 'viewWindowMode' => 'maximized' ],
					'trendlines'          => [
						'0' => [
							'type'            => 'linear',
							'color'           => \IPS\Settings::i()->spacious_graph_trendline_color,
							'lineWidth'       => \IPS\Settings::i()->spacious_graph_trendline_width,
							'opacity'         => '0.5',
							'showR2'          => 'false',
							'pointsVisible'   => 'false',
							'enableInteractivity' => 'false',
							'visibleInLegend' => 'false'
						]
					]
				]
			);


		}

		// Yell at them if they don't have Gallery installed, but do it in the template
		// They should not see this if the view check is good to go
		else
		{
			$topfourimages = [];
			$newestfourimages = [];
			$stats = [];
			$installed = 0;
			$nineties_images_chart = 0;
			$seven_days_images = 0;
		}

		/* Display */
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'spacious_gallerytrends.css', 'spaciouswidgets', 'admin' ) );
		return \IPS\Theme::i()->getTemplate( 'dashboard', 'spaciouswidgets' )->galleryStats($installed, $stats, $topfourimages, $newestfourimages, $nineties_images_chart, $seven_days_images );
	}

}