<?php
/**
 * @brief		Dashboard extension: memberStatistics
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Spacious Widgets
 * @since		16 Oct 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\spaciouswidgets\extensions\core\Dashboard;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Dashboard extension: memberStatistics
 */
class _memberStatistics
{
	/**
	 * Can the current user view this dashboard item?
	 *
	 * @return	bool
	 */
	public function canView()
	{
		return TRUE;
	}

	/**
	 * Return the block HTML show on the dashboard
	 *
	 * @return	string
	 */
	public function getBlock()
	{

	$memtrends = [];

	/* How many members? Just checks that the joined field is larger than zero and counts on that. */
	/* Really only a hedge against member extensions that fail leaving many broken member entries behind */
	/* Folded in name and email in Spacious 6 */
	/* OLD: ['joined > ?', 0] */

	$where = [ 'name<>? AND email<>? AND joined>?', '', '', 0 ];
	$memtrends['member_count'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $where)->first();

	/* Median member age - called here only on initial use based on default settings value */
	/* This code otherwise runs daily via task */

	if (! \IPS\Settings ::i()->spacious_median_memberage >= 0)
	{
		/* Insane attempt at average member ages */
		/* This will never be accurate - truthy at best */
		/* Can never trust the data coming in but still - a decent approximate to have. */

		$itsyourbirthdays = \IPS\Db::i()->select(
			'bday_year, COUNT(*) AS howmany', 'core_members', [
				[ 'core_members.bday_year >= ?', \IPS\Settings::i()->spacious_bdays_oldyear ],
				[ 'core_members.bday_year <= ?', \IPS\Settings::i()->spacious_bdays_youngyear ]
			], 'howmany DESC', NULL, 'bday_year'
		);

		$bdays = [ ];
		foreach ( $itsyourbirthdays as $thing )
		{
			$bdays[ $thing['bday_year'] ] = $thing['howmany'];
		}

		/* So that gets us $bdays[YEAR].COUNT - more better explained, that is an array of years */
		/* with the associated count of how many instances of that year  - so each year plus how */
		/* many birthdays in that year. Use this to plot out later, for now we'll need this to grind */
		/* for the median. */

		$year = date( 'Y' );
		$ages = [ ];
		$i = 0;
		foreach ( $bdays as $k => $v )
		{
			$how_old = $year - $k;
			while($v >= 1)
			{
				$ages[ $i ] = $how_old;
				$v = $v - 1;
			}
			$i++;
		}

		sort( $ages );

		/* I pronounce the following close enough for government work...*/
		$middle = floor( ( ( count( $ages ) ) - 1 ) / 2 );

		$median_age = $ages[ (int)$middle ];
		if ( is_nan( $median_age ) )
		{
			$median_age = 0;
		}

		// New method for updating settings circa 4.2.5?
		\IPS\Settings::i()->changeValues( array( 'spacious_median_memberage' => $median_age ) );

	}


	/* Members active over the last day, week, month */

		$whereday = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'PT24H' ) )->getTimeStamp();
		$whereweek = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P7D' ) )->getTimeStamp();
		$where14 = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P14D' ) )->getTimeStamp();
		$where21 = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P21D' ) )->getTimeStamp();
		$where28 = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P28D' ) )->getTimeStamp();
		$whereyear = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P1Y' ) )->getTimeStamp();
		$wheresixmonths = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P6M' ) )->getTimeStamp();
		$wherethreemonths = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P3M' ) )->getTimeStamp();
		$whereonemonth = 'core_members.last_visit>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P1M' ) )->getTimeStamp();


		// REBUILD FOR S14

		// Yesterday is needed to get the member count of those here today exclusively
		// Other vars can include up to and thru right now.

		$now = new \DateTime();
		$yearback = new \DateTime();
		$yearback->sub(new \DateInterval('P1Y'));
		$yearback->setTime(0,0,0); // This sets this at midnight 1y ago, and so on for the others.
		$yearback = $yearback->getTimeStamp();

		$ninetyback = new \DateTime();
		$ninetyback->sub(new \DateInterval('P90D'));
		$ninetyback->setTime(0,0,0);
		$ninetyback = $ninetyback->getTimeStamp();

		$weekback = new \DateTime();
		$weekback->sub(new \DateInterval('P7D'));
		$weekback->setTime(0,0,0);
		$weekback = $weekback->getTimeStamp();

		$yesterday = new \DateTime();
		$yesterday->sub(new \DateInterval('P1D'));
		$yesterday->setTime(23,59,59); // $yesterday is now yesterday at 11:59:59 pm. This is so we can grab just the today stuff.
		$yesterday = $yesterday->getTimeStamp();

		// This sets daily_members to the number of members who have visited since Midnight.
		$today_members =   ['core_members.last_visit>? AND name<>? AND email<>? AND joined>?', $yesterday, '', '', 0 ];
		$memtrends['today_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $today_members )->first();

		// These will include up to now.
		$week_members =  ['core_members.last_visit>? AND name<>? AND email<>? AND joined>?', $weekback, '', '', 0 ];
		$ninety_members = [ 'core_members.last_visit>? AND name<>? AND email<>? AND joined>?',  $ninetyback, '', '', 0 ];
		$year_members =  ['core_members.last_visit>? AND name<>? AND email<>? AND joined>?',  $yearback, '', '', 0 ];



		// We can assign the week members directly as no maths involved.
		$memtrends['week_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $week_members )->first();
		// Others for maths. NOTE logic problem with array - select count to var. Investigate maybe...

		$memtrends['ninety'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $ninety_members )->first();
		$memtrends['year'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $year_members )->first();
		// Maths for Zathrus.
		$memtrends['ninety_members'] = $memtrends['ninety'] - $memtrends['week_members'];
		if ($memtrends['ninety_members'] < 1)
		{
			$memtrends['ninety_members'] = 0;
		}
		$memtrends['oneyear_members'] = $memtrends['year'] - $memtrends['ninety'] ;
		if ($memtrends['oneyear_members'] < 1)
		{
			$memtrends['oneyear_members'] = 0;
		}


		$memtrends['the_inactive_members'] = $memtrends['member_count'] - $memtrends['year'];

		$memtrends['everyone_that_matters'] = $memtrends['year'];


		// Percents to make life easy, div by zero possible so...
		if ($memtrends['week_members'] > 0)
		{
			$memtrends['week_percent'] = round((($memtrends['week_members'] /$memtrends['member_count']))*100, 1);
		}
		else
		{
			$memtrends['week_percent'] = 0;
		}

		if ($memtrends['ninety_members'] > 0)
		{
			$memtrends['ninety_percent'] = round((($memtrends['ninety_members'] /$memtrends['member_count']))*100, 1);
		}
		else
		{
			$memtrends['ninety_percent'] = 0;
		}

		if ($memtrends['oneyear_members'] > 0)
		{
			$memtrends['oneyear_percent'] = round((($memtrends['oneyear_members'] /$memtrends['member_count']))*100, 1);
		}
		else
		{
			$memtrends['oneyear_percent'] = 0;
		}





		// I need members who have visted the last seven days, last 90 days, last year.
		// Last 90 needs to exclude last week, last year needs to exclude last 90 and last 7
		//

		$memtrends['daily_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $whereday )->first();
		$memtrends['weekly_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $whereweek )->first();
		$memtrends['14d_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $where14 )->first();
		$memtrends['21d_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $where21 )->first();
		$memtrends['28d_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $where28 )->first();
		$memtrends['1m_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $whereonemonth )->first();
		$memtrends['year_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $whereyear )->first();
		$memtrends['6m_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wheresixmonths )->first();
		$memtrends['3m_members'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherethreemonths )->first();

		if (\IPS\Settings::i()->spacious_members_inactive == 2) {
			$memtrends['inactive_members'] = $memtrends['year_members'] - $memtrends['6m_members'];
			$memtrends['active_members'] = $memtrends['6m_members'] - $memtrends['3m_members'];
			$memtrends['relic_members'] = $memtrends['member_count'] - $memtrends['year_members'];
		}
		elseif (\IPS\Settings::i()->spacious_members_inactive == 1) {
			$memtrends['inactive_members'] = $memtrends['6m_members'] - $memtrends['3m_members'];
			$memtrends['active_members'] = $memtrends['3m_members'] - $memtrends['1m_members'];
			$memtrends['relic_members'] = $memtrends['member_count'] - $memtrends['6m_members'];
		}

		else {
			$memtrends['inactive_members'] = $memtrends['3m_members'] - $memtrends['1m_members'];
			$memtrends['active_members'] = $memtrends['1m_members'] - $memtrends['weekly_members'];
			$memtrends['relic_members'] = $memtrends['member_count'] - $memtrends['3m_members'];
		}

		/* ********************************************** */

		/* Init RELICS Chart  - large for details dialog, includes ALL registered members */
		$chart = new \IPS\Helpers\Chart;

		$chart->addHeader( "Members", 'string' );
		$chart->addHeader( "Total", 'number' );


		if (\IPS\Settings::i()->spacious_members_inactive == 2) {
			$chart->addRow( [ "Regulars 3m", $memtrends['3m_members'] ] );
			$chart->addRow( [ "Members 6m-3m", $memtrends['active_members'] ] );
			$chart->addRow( [ "Members 1y-6m", $memtrends['inactive_members'] ] );
		}
		elseif (\IPS\Settings::i()->spacious_members_inactive == 1) {
			$chart->addRow( [ "Regulars 1m", $memtrends['1m_members'] ] );
			$chart->addRow( [ "Members 3m-1m", $memtrends['active_members'] ] );
			$chart->addRow( [ "Members 6m-3m", $memtrends['inactive_members'] ] );
		}
		else {
			$chart->addRow( [ "Regulars 1w", $memtrends['weekly_members'] ] );
			$chart->addRow( [ "Members 1m-1w", $memtrends['active_members'] ] );
			$chart->addRow( [ "Members 3m-1m", $memtrends['inactive_members'] ] );
		}

		$chart->addRow( [ "Relics", $memtrends['relic_members'] ] );


		$relics_chart = $chart->render( 'PieChart', [
				'height' => '320',
				'width' => '320',
				'pieHole' => '0.4',
				'fontSize' => '16',
				'pieSliceText' => 'none',
				'tooltip' => ['trigger' => 'focus'],
				'backgroundColor' => 'transparent',
				'enableInteractivity' => 'true',
				'legend' => ['position' => 'none'],
				'chartArea' => ['backgroundColor' => 'transparent', 'height' => '290', 'width' => '290'],
				'slices' => ['0'=>['color' => '#439edc'], '1'=>['color' => '#407ba2'], '2'=>['color' => '#5a6d79'], '3'=>['color' => '#2b2b2b'] ]
			]
		);


		/* ********************************************** */

		/* Init ACTIVES Chart */
		$chart = new \IPS\Helpers\Chart;

		$chart->addHeader( "Members", 'string' );
		$chart->addHeader( "Total", 'number' );


		if (\IPS\Settings::i()->spacious_members_inactive == 2) {
			$chart->addRow( [ "Regulars Now-3m", $memtrends['3m_members'] ] );
			$chart->addRow( [ "Active Members 3m-6m", $memtrends['active_members'] ] );
			$chart->addRow( [ "Members 6m-1y", $memtrends['inactive_members'] ] );
		}
		elseif (\IPS\Settings::i()->spacious_members_inactive == 1) {
			$chart->addRow( [ "Regulars Now-1m", $memtrends['1m_members'] ] );
			$chart->addRow( [ "Active Members 1m-3m", $memtrends['active_members'] ] );
			$chart->addRow( [ "Members 3m-6m", $memtrends['inactive_members'] ] );
		}
		else {
			$chart->addRow( [ "Regulars Now-1w", $memtrends['weekly_members'] ] );
			$chart->addRow( [ "Active Members 1w-1m", $memtrends['active_members'] ] );
			$chart->addRow( [ "Members 1m-3m", $memtrends['inactive_members'] ] );
		}

		$actives_chart = $chart->render( 'PieChart', [
				'height' => '170',
				'width' => '170',
				'pieHole' => '0.4',
				'backgroundColor' => 'transparent',
				'enableInteractivity' => 'true',
				'legend' => ['position' => 'none'],
				'chartArea' => ['backgroundColor' => 'transparent', 'height' => '150', 'width' => '150'],
				'slices' => ['0'=>['color' => '#439edc'], '1'=>['color' => '#407ba2'], '2'=>['color' => '#5a6d79'] ]
			]
		);


		/* Let's graph out new members over the last eight weeks! */

		$wherenewmembersweek = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P7D' ) )->getTimeStamp();
		$wherenewmembers_14 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P14D' ) )->getTimeStamp();
		$wherenewmembers_21 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P21D' ) )->getTimeStamp();
		$wherenewmembers_28 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P28D' ) )->getTimeStamp();
		$wherenewmembers_35 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P35D' ) )->getTimeStamp();
		$wherenewmembers_42 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P42D' ) )->getTimeStamp();
		$wherenewmembers_49 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P49D' ) )->getTimeStamp();
		$wherenewmembers_56 = 'core_members.joined>' . \IPS\DateTime::create()->sub( new \DateInterval( 'P56D' ) )->getTimeStamp();

		$memtrends['newmembers_week'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembersweek )->first();
		$memtrends['newmembers_14'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_14 )->first();
		$memtrends['newmembers_21'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_21 )->first();
		$memtrends['newmembers_28'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_28 )->first();
		$memtrends['newmembers_35'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_35 )->first();
		$memtrends['newmembers_42'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_42 )->first();
		$memtrends['newmembers_49'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_49 )->first();
		$memtrends['newmembers_56'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', $wherenewmembers_56 )->first();

		$memtrends['newmembers_14count'] = $memtrends['newmembers_14'] - $memtrends['newmembers_week'];
		$memtrends['newmembers_21count'] = $memtrends['newmembers_21'] - $memtrends['newmembers_14'];
		$memtrends['newmembers_28count'] = $memtrends['newmembers_28'] - $memtrends['newmembers_21'];
		$memtrends['newmembers_35count'] = $memtrends['newmembers_35'] - $memtrends['newmembers_28'];
		$memtrends['newmembers_42count'] = $memtrends['newmembers_42'] - $memtrends['newmembers_35'];
		$memtrends['newmembers_49count'] = $memtrends['newmembers_49'] - $memtrends['newmembers_42'];
		$memtrends['newmembers_56count'] = $memtrends['newmembers_56'] - $memtrends['newmembers_49'];

		/* Now fill a graph array */

		$newmembers['7'] = $memtrends['newmembers_week'];
		$newmembers['14'] = $memtrends['newmembers_14count'];
		$newmembers['21'] = $memtrends['newmembers_21count'];
		$newmembers['28'] = $memtrends['newmembers_28count'];
		$newmembers['35'] = $memtrends['newmembers_35count'];
		$newmembers['42'] = $memtrends['newmembers_42count'];
		$newmembers['49'] = $memtrends['newmembers_49count'];
		$newmembers['56'] = $memtrends['newmembers_56count'];



		/* ********************************************** */

		/* Init New Members Chart */
		$chart = new \IPS\Helpers\Chart;

		$chart->addHeader( "Week", 'number' );
		$chart->addHeader( "Members", 'number' );

		foreach ($newmembers as $k => $v) {
			$chart->addRow( [ $k, $v ] );
		}


		$newmembers_chart = $chart->render( 'LineChart', [
				'height' => '40',
				'backgroundColor' => 'transparent',
				'enableInteractivity' => 'false',
				'legend' => ['position' => 'none'],
				'bar' => ['groupWidth' => '100%'],
				'chartArea' => ['backgroundColor' => 'transparent', 'height' => '40', 'width' => '80', 'stroke' => '#f5f5f5'],
				'reverseCategories' => 'true',
				'hAxis' => [ 'gridlines' => [ 'count' => "0"], 'baselineColor' => "#f5f5f5" ],
				'vAxis' => [ 'gridlines' => [ 'count' => "0"], 'baselineColor' => "#f5f5f5" ],
				'trendlines' => [ '0' => [
					'type' => 'linear',
					'color'=> 'green',
					'lineWidth' => '2',
					'opacity' => '0.3',
					'showR2' => 'false',
					'pointsVisible' => 'false',
					'visibleInLegend' => 'false'] ]
			]
		);

		/* Rows already set, no reason we can't spin off another larger chart - just cast to a different var! */

		$newmembers_chart_large = $chart->render( 'LineChart', [
				'height' => '175',
				'width' => '400',
				'pointSize' => '0',
				'backgroundColor' => 'transparent',
				'enableInteractivity' => 'true',
				'legend' => ['position' => 'none'],
				'lineWidth' => '5',
				'chartArea' => ['backgroundColor' => 'transparent', 'height' => '175', 'width' => '350', 'stroke' => '#fff'],
				'reverseCategories' => 'true',
				'hAxis' => [ 'gridlines' => [ 'count' => "0"], 'baselineColor' => "#fff" ],
				'vAxis' => [ 'gridlines' => [ 'count' => "0"], 'baselineColor' => "#fff" ],
				'trendlines' => [ '0' => [
					'type' => 'linear',
					'color'=> 'green',
					'lineWidth' => '3',
					'opacity' => '0.3',
					'showR2' => 'false',
					'pointsVisible' => 'false',
					'visibleInLegend' => 'false'] ]
			]
		);




		/* Display yonder member junk */

		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'spaciouswidgets_ms.css', 'spaciouswidgets', 'admin' ) );

		return \IPS\Theme::i()->getTemplate( 'dashboard', 'spaciouswidgets' )->memberStatistics($memtrends, $newmembers, $newmembers_chart, $newmembers_chart_large, $relics_chart, $actives_chart);
	}
}