<?php
/**
 * @brief		Dashboard extension: systemOverview
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Spacious Widgets
 * @since		05 Sep 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\spaciouswidgets\extensions\core\Dashboard;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Dashboard extension: systemOverview
 */
class _systemOverview
{
	/**
	* Can the current user view this dashboard item?
	*
	* @return	bool
	*/
	public function canView()
	{
		return TRUE;
	}

	/** 
	 * Return the block HTML show on the dashboard
	 *
	 * @return	string
	 */
	public function getBlock()
	{

	$rootpath = \IPS\ROOT_PATH . "/";

	$started = \IPS\Settings::i()->getFromConfGlobal('board_start');

	/* TASK: spaciousSystemSize. Runs once a day and parks result in settings. */
	/* Template is grabbing from settings so for convenience we can just duplicate the code here and run on install */
	/* otherwise it will display zero until the task runs by itself */

	if (\IPS\Settings::i()->spacious_syso_totalspace == 0)
	{
		$crawl = new \RecursiveDirectoryIterator( $rootpath, \FilesystemIterator::SKIP_DOTS );
		$filesys = 0;
		foreach ( new \RecursiveIteratorIterator(
			$crawl, \RecursiveIteratorIterator::SELF_FIRST, \RecursiveIteratorIterator::CATCH_GET_CHILD
		) as $filename => $cur )
		{
			$filesize = $cur->getSize();
			$filesys += $filesize;
		}

		\IPS\Db::i()->update(
			'core_sys_conf_settings', [ 'conf_value' => $filesys ], [ 'conf_key=?', 'spacious_syso_totalspace' ]
		);
		\IPS\Settings::i()->spacious_syso_totalspace = $filesys;

		// Unset datastore to get the new value in there.
		unset( \IPS\Data\Store::i()->settings );
	}

		$filesys = \IPS\Settings::i()->spacious_syso_totalspace;

	/* Operating system and freespace left on the disk which is somewhat important... */
	$opsys = [];
	$opsys['s'] = php_uname("s");
	$opsys['v'] = php_uname("v");
	$opsys['m'] = php_uname("m");

	/* Rough get for Debian and RedHat/Centos */
	if ( @file_exists('/etc/redhat-release') )
	{
		$opsys['s'] = file_get_contents('/etc/redhat-release');
	}

	if ( @file_exists('/etc/debian_version') )
	{
		$opsys['s'] = file_get_contents('/etc/debian_version');
	}

	$disk = [];
	$disk['size'] = disk_total_space("/");
	$disk['free'] = disk_free_space($rootpath);
	$disk['percentfree'] = round((($disk['free'] /$disk['size']))*100, 1);

	$servername = gethostname();

	/* DATABASE */

		/* DB Size... */

		$ipsdb = \IPS\Settings::i()->getFromConfGlobal('sql_database');
		$database_timezone =  \IPS\Db::i()->query("SELECT @@system_time_zone")->fetch_row();
		$datasize = \IPS\Db::i()->query("SELECT SUM(data_length)
FROM information_schema.tables
WHERE table_schema ='". $ipsdb. "'")->fetch_row();

		$indexsize = \IPS\Db::i()->query("SELECT SUM(index_length)
FROM information_schema.tables
WHERE table_schema ='". $ipsdb. "'")->fetch_row();

		$databasesize = $datasize[0] + $indexsize[0];

		$dbov['totalsize'] = $databasesize;
		$dbov['datasize'] = $datasize[0];
		$dbov['indexsize'] = $indexsize[0];
		$dbov['totalsizereadable'] = \IPS\Output\Plugin\Filesize::humanReadableFilesize($databasesize, TRUE);
		$dbov['datasizereadable'] = \IPS\Output\Plugin\Filesize::humanReadableFilesize($datasize[0], TRUE);
		$dbov['indexsizereadable'] = \IPS\Output\Plugin\Filesize::humanReadableFilesize($indexsize[0], TRUE);
		$dbov['timezone'] = $database_timezone[0];

		/* Grab some basic stats, parse into usable template variables */
		$mysqlstats  = explode('  ', \IPS\Db::i()->stat());

		preg_match('/(?<=: ).*/s', $mysqlstats[0], $uptime);
		preg_match('/(?<=: ).*/s', $mysqlstats[7], $qpsa);
		preg_match('/(?<=: ).*/s', $mysqlstats[1], $threads);
		preg_match('/(?<=: ).*/s', $mysqlstats[3], $slowq);
		$qpsa[0] = round($qpsa[0], 2,  PHP_ROUND_HALF_UP);
		$rightnow = new \DateTime('@0');
		$timeup = new \DateTime("@$uptime[0]");
		$dbuptime = [];
		$dbuptime['days'] = $rightnow->diff($timeup)->format('%a');
		$dbuptime['hours'] = $rightnow->diff($timeup)->format('%h');
		$dbuptime['minutes'] = $rightnow->diff($timeup)->format('%i');

		$thing = \IPS\Db::i()->query("SHOW VARIABLES LIKE 'ft_min_word_len'")->fetch_row();
		list($null, $ftminlen) = $thing;

		$thing = \IPS\Db::i()->query("SHOW VARIABLES LIKE 'innodb_ft_min_token_size'")->fetch_row();
		list($null, $innomintoken) = $thing;

		$thing = \IPS\Db::i()->query("SHOW VARIABLES LIKE 'innodb_buffer_pool_instances'")->fetch_row();
		list($null, $innobpinstances) = $thing;

		$thing = \IPS\Db::i()->query("SHOW VARIABLES LIKE 'innodb_buffer_pool_size'")->fetch_row();
		list($null, $innobpsize) = $thing;

		$innobpsize = \IPS\Output\Plugin\Filesize::humanReadableFilesize($innobpsize, TRUE);

		$search_table = \IPS\Db::i()->getTableDefinition('core_search_index');
		$search_db_engine = $search_table['engine'];



	/* Now that we have both file system size and the db size, let's combo them so we can match against quota, or just provide a footprint total */

		$ipsfootprint = $filesys + $dbov['totalsize'];
		$disk['ipsfootprint'] = $ipsfootprint;
		$disk['ipspercent'] = round((($ipsfootprint /$disk['size']))*100, 1);



		$quota['quotareadable'] = 0;
		$quota['quotaremainingreadable'] = 0;

		/* If CIC is on let's calc some stuff */

		if (\IPS\CIC)
		{
			$quotakb = (\IPS\Settings ::i()->spacious_syso_cic_quota) * 1000 * 1000 * 1000;
			$quotaremaining = $quotakb - $ipsfootprint;
			$quotakb = \IPS\Output\Plugin\Filesize::humanReadableFilesize($quotakb);
			$quotaremaining = \IPS\Output\Plugin\Filesize::humanReadableFilesize($quotaremaining);
			$quota['quotareadable'] = $quotakb;
			$quota['quotaremainingreadable'] = $quotaremaining;
		}


	/* Why not load averages... */
		$load = [];
		if ($opsys['s'] == "Windows NT")
		{
		$load[0] = 0;
		$load[1] = 0;
		$load[2] = 0;
			}
		else
		{
			$load = sys_getloadavg();
		}

    /* CPU Info? Mem Info?*/

		/* If using a win box for dev, option out of this stuff */
		if ($opsys['s'] == "Windows NT") {
			$cores = 0;
			$cpu[0] = "Undetected";
			$memory = [];
			$memory['total'] = "49284816000";
			$memory['free'] = "Undetected";
			$memory['available'] = "15000000000";
			$memory['percentleft'] = 54;
			$cpuinfo[0] = "Undetected";
			$meminfo[0] = "Undetected";

		}
		else {

			$cores = 0;

			if ( @file_exists('/proc/cpuinfo') )
			{
				$cpuinfo = file_get_contents('/proc/cpuinfo');
				preg_match_all('/^processor/m', $cpuinfo, $cpucount);
				preg_match('/(?<=model name	: ).*?(?=stepping)/s', $cpuinfo, $cpu);
				$cores = count($cpucount[0]);

				/* And let's make cpuinfo entirely readable for the full details link, line returns, etc. */
				$cpuinfo = preg_split('/\r\n|\n|\r/', $cpuinfo);

			}
			else
			{
			$cpu[0] = "Undetected";
			$cpuinfo[0] = "Somehow Inaccessible";

			}

			/* Not as simple as just pulling available, older Linux builds don't have available so have to pull free as well */
			/* Note the difference between free and available. Free is the exact amount of memory not in use. */
			/* Available is memory that is being used but is free to take and use by the system whenever it wants to */

			$memory = [];
			if ( @file_exists('/proc/meminfo') )
			{
				$meminfo = file_get_contents('/proc/meminfo');
				preg_match('/(?<=MemTotal: ).*?(?= kB)/s', $meminfo, $memoryproc0);
				preg_match('/(?<=MemFree: ).*?(?= kB)/s', $meminfo, $memoryproc1);
				preg_match('/(?<=MemAvailable: ).*?(?= kB)/s', $meminfo, $memoryproc2);

				/* Whatever, I'm over trying to get this "right" - 1024 for everyone! */
				$memory['total'] = $memoryproc0[0] * 1024;
				$memory['free'] = $memoryproc1[0] * 1024;
				$memory['available'] = $memoryproc2[0] * 1024;

				if ($memoryproc2[0] > 0)
				{
					$memory['percentleft'] = round((($memory['available'] /$memory['total']))*100, 1);
				}
				elseif ($memoryproc1[0] > 0)
				{
					$memory['percentleft'] = round((($memory['free'] /$memory['total']))*100, 1);
				}

				/* And let's make meminfo entirely readable for the full details link, line returns, etc. */
				$meminfo = preg_split('/\r\n|\n|\r/', $meminfo);
			}
			else
			{
				$memory['total'] = "Undetected";
				$memory['free'] = "Undetected";
				$memory['available'] = "Undetected";
				$memory['percentleft'] = "Undetected";
				$meminfo[0] = "Undetected";
			}

		}

	/* Since we have cpuinfo, allegedly, let's div that load */
	/* Otherwise just multiply by 100 for single cores */

		if ($cores > 1) {
			$load[0] = ($load[0]/$cores)*100;
			$load[1] = ($load[1]/$cores)*100;
			$load[2] = ($load[2]/$cores)*100;
		}
		else {
			$load[0] = ($load[0])*100;
			$load[1] = ($load[1])*100;
			$load[2] = ($load[2])*100;
		}

		// Safety for percents north of 100 for the background bar
		// Ternary this pls.

		if ($load[0] > 100)
		{
			$load[3] = 100;
		}
		else
		{
			$load[3] = $load[0];
		}


		if ($opsys['s'] == "Windows NT") {
			$load[3] = 65;
		}

		/* Now round them all... */

		$load[0] = round( $load[0], 1, PHP_ROUND_HALF_UP);
		$load[1] = round( $load[1], 1, PHP_ROUND_HALF_UP);
		$load[2] = round( $load[2], 1, PHP_ROUND_HALF_UP);
		$load[3] = round( $load[3], 1, PHP_ROUND_HALF_UP);

		/* Server Uptime */

		$uptime = 0;
		$idlepercent = 0;

		if ( @file_exists('/proc/uptime') )
		{
			$serverup = explode( ' ', file_get_contents( '/proc/uptime' ) );
			$up = round( $serverup[0] );
			$idle = round( $serverup[1] );

			$rightnow = new \DateTime( '@0' );
			$timeup = new \DateTime( "@$up" );
			$uptime = [ ];
			$uptime['days'] = $rightnow->diff( $timeup )->format( '%a' );
			$uptime['hours'] = $rightnow->diff( $timeup )->format( '%h' );
			$uptime['minutes'] = $rightnow->diff( $timeup )->format( '%i' );

			/* Idle time to percent - idle is in seconds but multiplied by cores so... */
			$idlepercent = ( $idle / $up ) * 100;
			if ( $cores > 1 )
			{
				$idlepercent = round($idlepercent / $cores, 1, PHP_ROUND_HALF_UP);
			}

		}


		/* Server Geolocation */
		$geolocation	= NULL;
		$map			= NULL;
		try
		{
			$geolocation = \IPS\GeoLocation::getByIp( $_SERVER['SERVER_ADDR'] );
			// Limited to 640 wide with current IPS method. Overload later if no movement on this by IPS
			$map = $geolocation->map()->render( 803, 135, 0.6 );
		}
		catch ( \Exception $e ) {}





		/* That's it. Fancy formatting for low diskspace and such done in the template */

		/* PUSH OUT THE TEMPLATE */
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'spaciouswidgets.css', 'spaciouswidgets', 'admin' ) );
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'spacious_systemoverview.css', 'spaciouswidgets', 'admin' ) );
		return \IPS\Theme::i()->getTemplate( 'dashboard', 'spaciouswidgets' )->systemOverview($rootpath, $opsys, $disk, $started, $load, $cores, $cpu, $memory, $cpuinfo, $meminfo, $servername, $quota, $uptime, $idlepercent, $dbov, $dbuptime, $slowq, $threads, $qpsa, $ftminlen, $innomintoken, $innobpinstances, $innobpsize, $search_db_engine, $geolocation, $map );
	}
}