<?php
/**
 * @brief		spaciousAlexa Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	spaciouswidgets
 * @since		17 Mar 2017
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\spaciouswidgets\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * spaciousAlexa Task
 */
class _spaciousAlexa extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{

		/* Grab Setting Vars */

		$now = time();
		$globalrank = \IPS\Settings ::i()->spacious_alexa_global_rank;
		$countrycode = \IPS\Settings ::i()->spacious_alexa_countrycode;
		$countryrank = \IPS\Settings ::i()->spacious_alexa_country_rank;
		$globalrankdirection = \IPS\Settings ::i()->spacious_alexa_global_rank_direction;
		$countryrankdirection = \IPS\Settings ::i()->spacious_alexa_country_rank_direction;

		$rawurl = \IPS\Http\Url::baseUrl();

		if (mb_strpos($rawurl,"localhost"))
		{
			$url["host"] = 'all-astronauts.com';
		}
		else
		{
			$url = parse_url( $rawurl );
		}


		/* Localhost or on sites with no data available we need to work around those null results with zeros and deal in template */

		$xml = simplexml_load_file('https://data.alexa.com/data?cli=10&dat=snbamz&url='.$url["host"]);

		/* Well then, if allow_fopen is off simplexml won't work. Skip this for now, CURL around it later */


		/* What if the site has no tracking data? XML returned will be effectively empty... */
		/* So if vars are set, great, otherwise set them to 0 */

		$newglobalrank = isset($xml->SD[1]->POPULARITY)?$xml->SD[1]->POPULARITY->attributes()->TEXT:0;
		$newcountryrank = isset($xml->SD[1]->COUNTRY)?$xml->SD[1]->COUNTRY->attributes()->RANK:0;
		$newcountrycode = isset($xml->SD[1]->COUNTRY)?$xml->SD[1]->COUNTRY->attributes()->NAME:0;

		/* Set Direction Flags and Change Vars as needed but account for zeros. Just text newglobalrank and go from there... */

		if ($newglobalrank != 0) {

			if ( $newglobalrank > $globalrank )
			{
				$globalrankdirection = 2;
				$globalrank = $newglobalrank;
			}
			elseif ( ( $newglobalrank < $globalrank ) && ( $newglobalrank != 0 ) )
			{
				$globalrankdirection = 1;
				$globalrank = $newglobalrank;
			}
			else
			{
				$globalrankdirection = 0;
			}

			if ( $newcountryrank > $countryrank )
			{
				$countryrankdirection = 2;
				$countryrank = $newcountryrank;
			}
			elseif ( ( $newcountryrank < $countryrank ) && ( $newcountryrank != 0 ) )
			{
				$countryrankdirection = 1;
				$countryrank = $newcountryrank;
			}
			else
			{
				$countryrankdirection = 0;
			}

			/* Have to check countrycode just to be safe */

			if ( $newcountrycode !== $countrycode )
			{
				$countrycode = $newcountrycode;
			}

		}


		/* Now write the settings back even on the off-chance nothing has changed since we are in the loop anyways */
		/* Hey - if this looks ugly don't look at me - same thing the form helper does, just in a foreach loop */

		\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => $countryrank ], [ 'conf_key=?', 'spacious_alexa_country_rank' ] );
		\IPS\Settings::i()->spacious_alexa_country_rank	= $countryrank;

		\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => $globalrank ], [ 'conf_key=?', 'spacious_alexa_global_rank' ] );
		\IPS\Settings::i()->spacious_alexa_global_rank = $globalrank;

		\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => $countrycode ], [ 'conf_key=?', 'spacious_alexa_countrycode' ] );
		\IPS\Settings::i()->spacious_alexa_countrycode = $countrycode;

		\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => $globalrankdirection ], [ 'conf_key=?', 'spacious_alexa_global_rank_direction' ] );
		\IPS\Settings::i()->spacious_alexa_global_rank_direction = $globalrankdirection;

		\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => $countryrankdirection ], [ 'conf_key=?', 'spacious_alexa_country_rank_direction' ] );
		\IPS\Settings::i()->spacious_alexa_country_rank_direction = $countryrankdirection;

		/* Last write is the standard timestamp */

		\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => $now ], [ 'conf_key=?', 'spacious_alexa_datetime' ] );
		\IPS\Settings::i()->spacious_alexa_datetime	= $now;


		/* Settings are set in DB and right here but if you want them to be set in the cache I think you have to drop that junk to rebuild it with the new data */

		unset( \IPS\Data\Store::i()->settings );

		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}