<?php
/**
 * @brief		spaciousMedianAge Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	spaciouswidgets
 * @since		12 Nov 2017
 */

namespace IPS\spaciouswidgets\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * spaciousMedianAge Task
 */
class _spaciousMedianAge extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{


			/* Insane attempt at average member ages */
			/* This will never be accurate - truthy at best */
			/* Can never trust the data coming in but still - a decent approximate to have. */

			$itsyourbirthdays = \IPS\Db::i()->select(
				'bday_year, COUNT(*) AS howmany', 'core_members', [
					[ 'core_members.bday_year >= ?', \IPS\Settings::i()->spacious_bdays_oldyear ],
					[ 'core_members.bday_year <= ?', \IPS\Settings::i()->spacious_bdays_youngyear ]
				], 'howmany DESC', NULL, 'bday_year'
			);

			$bdays = [ ];
			foreach ( $itsyourbirthdays as $thing )
			{
				$bdays[ $thing['bday_year'] ] = $thing['howmany'];
			}

			/* So that gets us $bdays[YEAR].COUNT - more better explained, that is an array of years with the associated count of how */
			/* many instances of that year  - so each year plus how many birthdays in that year. Use this to plot out later, for now */
			/* we'll need to grind for the median. */

			$year = date( 'Y' );
			$ages = [ ];
			$i = 0;
			foreach ( $bdays as $k => $v )
			{
				$how_old = $year - $k;
				while($v >= 1)
				{
					$ages[ $i ] = $how_old;
					$v = $v - 1;
				}
				$i++;
			}

			sort( $ages );

			/* I pronounce the following close enough for government work...*/
			$middle = floor( ( ( count( $ages ) ) - 1 ) / 2 );

			$median_age = $ages[ (int)$middle ];
			if ( is_nan( $median_age ) )
			{
				$median_age = 0;
			}

		// New method for updating settings circa 4.2.5?
		\IPS\Settings::i()->changeValues( array( 'spacious_median_memberage' => $median_age ) );

		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}